package cz.muni.fi.pa165.icehockeymanager.facades;

import cz.muni.fi.pa165.icehockeymanager.dto.GameCreateDTO;
import cz.muni.fi.pa165.icehockeymanager.dto.TeamCreateDTO;

/**
 * Facade with functionality for the league manager,
 * such as managing games and teams in the league.
 */
public interface LeagueManagerFacade {

    /**
     * Creates and plans a new game.
     *
     * @param gameCreateDTO DTO object for when game is created (without id)
     */
    void scheduleGame(GameCreateDTO gameCreateDTO);

    /**
     * Removes game from the schedule.
     * @param gameId ID of the game to be removed.
     */
    void dropGame(long gameId);

    /**
     * Updates the game score and also the winner if one is determined by the new score.
     * @param gameId ID of the game to be updated.
     * @param homeTeamScore the new score of the home team.
     * @param awayTeamScore the new score of the away team.
     */
    void updateGameScore(long gameId, int homeTeamScore, int awayTeamScore);

    /**
     * Sets the winner of a game.
     * @param gameId ID of the game of which the winner is set.
     * @param teamId ID of the winner team.
     */
    void setGameWinner(long gameId, long teamId);

    /**
     * Add new team to the league.
     * @param teamCreateDTO DTO of the new team to be created.
     */
    void createNewTeam(TeamCreateDTO teamCreateDTO);

}
