package cz.muni.fi.pa165.icehockeymanager.facades;

import cz.muni.fi.pa165.icehockeymanager.dto.GameCreateDTO;
import cz.muni.fi.pa165.icehockeymanager.dto.TeamCreateDTO;
import cz.muni.fi.pa165.icehockeymanager.exceptions.UnknownGameException;
import cz.muni.fi.pa165.icehockeymanager.exceptions.UnknownTeamException;
import cz.muni.fi.pa165.icehockeymanager.model.Game;
import cz.muni.fi.pa165.icehockeymanager.model.Team;
import cz.muni.fi.pa165.icehockeymanager.services.BeanMappingService;
import cz.muni.fi.pa165.icehockeymanager.services.GameService;
import cz.muni.fi.pa165.icehockeymanager.services.TeamService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class LeagueManagerFacadeImpl implements LeagueManagerFacade {

    private final TeamService teamService;
    private final BeanMappingService beanMappingService;
    private final GameService gameService;

    @Autowired
    public LeagueManagerFacadeImpl(TeamService teamService, BeanMappingService beanMappingService, GameService gameService) {
        this.teamService = teamService;
        this.beanMappingService = beanMappingService;
        this.gameService = gameService;
    }

    @Override
    public void scheduleGame(GameCreateDTO gameCreateDTO) {
        Game game = beanMappingService.mapTo(gameCreateDTO, Game.class);
        gameService.createGame(game);
        teamService.addHomeGame(game.getHomeTeam(), game);
        teamService.addAwayGame(game.getAwayTeam(), game);
    }

    @Override
    public void dropGame(long gameId) {
        Game game = gameService.findGame(gameId).orElseThrow(
                () -> new UnknownGameException("Game not found")
        );
        gameService.deleteGame(game);
        teamService.removeHomeGame(game.getHomeTeam(), game);
        teamService.removeAwayGame(game.getAwayTeam(), game);
    }

    @Override
    public void updateGameScore(long gameId, int homeTeamScore, int awayTeamScore) {
        Game game = gameService.findGame(gameId).orElseThrow(
                () -> new UnknownGameException("Game not found")
        );
        gameService.updateGameScore(game, homeTeamScore, awayTeamScore);
    }

    @Override
    public void setGameWinner(long gameId, long teamId) {
        Game game = gameService.findGame(gameId).orElseThrow(
                () -> new UnknownGameException("Game not found")
        );
        Team team = teamService.findTeam(teamId).orElseThrow(
                () -> new UnknownTeamException("Team not found")
        );
        gameService.setGameWinner(game, team);
    }

    @Override
    public void createNewTeam(TeamCreateDTO teamCreateDTO) {
        Team team = beanMappingService.mapTo(teamCreateDTO, Team.class);
        teamService.createTeam(team);
    }
}
