package cz.muni.fi.pa165.icehockeymanager.services;

import com.github.dozermapper.core.Mapper;
import cz.muni.fi.pa165.icehockeymanager.dto.GameDTO;
import cz.muni.fi.pa165.icehockeymanager.dto.TeamDTO;
import cz.muni.fi.pa165.icehockeymanager.model.Game;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class BeanMappingServiceImpl implements BeanMappingService {

    private final Mapper dozer;

    @Autowired
    public BeanMappingServiceImpl(Mapper dozer) {
        this.dozer = dozer;
    }

    public <T> List<T> mapTo(Collection<?> objects, Class<T> mapToClass) {
        List<T> mappedCollection = new ArrayList<>();
        for (Object object : objects) {
            mappedCollection.add(mapTo(object, mapToClass));
        }
        return mappedCollection;
    }

    public <T> T mapTo(Object u, Class<T> mapToClass) {
        if (u == null)
            return null;
        return dozer.map(u, mapToClass);
    }

    public GameDTO mapTo(Game u, Class<GameDTO> mapToClass) {
        if (u == null)
            return null;
        GameDTO mappedObject = dozer.map(u, mapToClass);
        TeamDTO winner = mapTo(u.getWinner().orElse(null), TeamDTO.class);
        mappedObject.setWinner(winner);
        return mappedObject;
    }

    @Override
    public List<GameDTO> mapCollection(Collection<Game> objects, Class<GameDTO> mapToClass) {
        return objects.stream().map((game) -> mapTo(game, GameDTO.class)).collect(Collectors.toList());
    }

    public Mapper getMapper() {
        return dozer;
    }
}